__Autor__ = "Luiz Fernando"
__Empresa__ = "Sig Informatica Ltda"
__Site__ = "http://www.siginformatica.com.br"
import re


class Cnpj:
    def __init__(self, cnpj, forcar=False):
        """
        Classe para testar CNPJ
        """
        self.cnpj = cnpj
        self.status = False
        self.formatado = ''
        self.digitos = ''
        self.forcar = forcar

    # Função para remover caracteres especiais do CNPJ
    def _remove_edicao(self):
        self.cnpj = re.sub(r'[^a-zA-Z0-9]', '', self.cnpj).upper()

    def remove_edicao(self):
        self._remove_edicao()
        if not self._verificar_tamanho():
            return {'status': False}
        return {'status': True, 'cnpj': self.cnpj}

    # Função para formatar o CNPJ (XX.XXX.XXX/XXXX-XX)
    def _formatar(self):
        self.formatado = f"{self.cnpj[:2]}.{self.cnpj[2:5]}.{self.cnpj[5:8]}/{self.cnpj[8:12]}-{self.cnpj[12:]}"

    def formatar(self):
        self._remove_edicao()
        if not self._verificar_tamanho():
            return {'status': False}
        self._formatar()
        return {'status': True, 'cnpj_formatado': self.formatado}

    # Função para calcular o dígito verificador
    def _calcular_digitos(self):
        # Mapeia letras para números (A=1, B=2, ..., Z=26)
        def valor_char(char):
            if char.isdigit():
                return int(char)
            return ord(char.upper()) - ord('A') + 1

        pesos1 = [5, 4, 3, 2, 9, 8, 7, 6, 5, 4, 3, 2]
        pesos2 = [6, 5, 4, 3, 2, 9, 8, 7, 6, 5, 4, 3, 2]

        # Calculando o primeiro dígito
        soma1 = sum(valor_char(self.cnpj[i]) * pesos1[i] for i in range(12))
        digito1 = 11 - (soma1 % 11)
        digito1 = 0 if digito1 >= 10 else digito1

        # Calculando o segundo dígito
        cnpj_base_com_digito1 = self.cnpj[:-2] + str(digito1)
        soma2 = sum(valor_char(cnpj_base_com_digito1[i]) * pesos2[i] for i in range(13))
        digito2 = 11 - (soma2 % 11)
        digito2 = 0 if digito2 >= 10 else digito2

        self.digitos = str(digito1) + str(digito2)
        return self.digitos

    def _verificar_tamanho(self):
        # Verifica se o CNPJ possui 12 caracteres antes do dígito (novo formato)
        if len(self.cnpj) > 14:
            status = False
            return status
        if len(self.cnpj) < 14:
            while len(self.cnpj) < 14:
                self.cnpj = '0' + self.cnpj
        status = True
        return status

    def validar(self):
        self._remove_edicao()
        if not self._verificar_tamanho():
            return {'status': False}

        # Calcula o dígito verificador
        self._calcular_digitos()

        # Verifica se o dígito do CNPJ é válido
        if self.forcar:
            self.cnpj = self.cnpj[:-2] + self.digitos
            self.status = True
        else:
            digito_informado = self.cnpj[-2:]
            if digito_informado == self.digitos:
                self.status = True
            else:
                self.status = False
                self.formatado = self.digitos = self.cnpj = ''
        if self.status:
            self._formatar()
        # Retorna o CNPJ formatado, o CNPJ sem formatação, o status e o dígito calculado
        return {
            "status": self.status,
            "cnpj_formatado": self.formatado,
            "cnpj": self.cnpj,
        }

